function log = f_osc_16()

% Load model
HA = f_osc_16_ha();

dim = 18;
% options
Zcenter = zeros(dim,1);
Zcenter(1) = 0.25;
Zdelta = zeros(dim,1);
Zdelta(1) = 0.05;
Zdelta(2) = 0.1;

options.R0 = zonotope([Zcenter,diag(Zdelta)]); %initial state for reachability analysis
options.x0 = center(options.R0); %initial state for simulation

options.startLoc = 3; %initial location
options.finalLoc = 0; %0: no final location
options.tStart = 0; %start time
options.tFinal = 4;

options.taylorTerms = 4;
options.zonotopeOrder = 100;
% options.polytopeOrder = 1;

% options.errorOrder=1.2;
% options.maxProjectionError=0.1;
options.reductionTechnique = 'girard';
options.isHyperplaneMap = 0;
options.enclosureEnables = [ 3 5 ];
options.originContained = 0;
options.guardIntersect = 'polytope';

%set input:
loc = get(HA,'location');
nrOfLoc = length(loc);
for i=1:nrOfLoc
    options.uLoc{i} = 0; % input for simulation
    options.uLocTrans{i} = options.uLoc{i}; % input center for reachability analysis
    options.Uloc{i} = zonotope(0); % input deviation for reachability analysis
    options.timeStepLoc{i} = 0.01;
end

% options.uLoc{1} = interval([0], [0]); % no input
% options.uLocTrans{1} = options.uLoc{1};
% options.Uloc{1} = zonotope(0);


dim = length(Zcenter);
sim = 0;
reacha = 1;
vis = 1;

% Simulation --------------------------------------------------------------

if sim
    N = 50;
    tic;
    for i=1:N
        %set initial state, input
        if i == 1
            %simulate center
            options.x0 = center(options.R0);
        elseif i < 5
            % simulate extreme points
            options.x0 = randPointExtreme(options.R0);
        else
            % simulate random points
            options.x0 = randPoint(options.R0);
        end 

        %simulate hybrid automaton
        HAsim = simulate(HA,options);
        simRes{i} = get(HAsim,'trajectory');
    end
    toc;
    disp(['Time needed for the simulation: ', num2str(toc)]);

    % Visualization -------------------------------------------------------
    figure 
    hold on
    box on
    options.projectedDimensions = [dim 1];
    options.plotType = {'b','m','g'};
    plotFilled(options.R0,options.projectedDimensions,'w','EdgeColor','k'); %plot initial set
    for i = 1:length(simRes)
       for j = 1:length(simRes{i}.x)
           plot(simRes{i}.x{j}(:,options.projectedDimensions(1)), ...
                simRes{i}.x{j}(:,options.projectedDimensions(2)),'k'); 
       end
    end
    xlabel('t');
    ylabel('x1');
end


% tmp
figure 

% Reachability ------------------------------------------------------------
if reacha
    tic;
    [HA] = reach(HA,options);
    reachabilityT = toc;
    disp(['Time needed for reachability analysis: ', num2str(reachabilityT)]);

    % Verification --------------------------------------------------------
    
    Rset = get(HA, 'reachableSet');
    Rset = Rset.OT;
    
    %spec = findSafetyProperties([-0.0061322  0.064531 -0.00069603 -0.062226 -0.0035048 0.047228],Rset);
    
    %tic;
    %safe = verifySafetyProperties(spec, Rset);
    %verificationT = toc;
    
    %if safe == ConfigState.Safe
    %    disp('Verification result: SAFE');
    %end
    
    %disp(['Time needed for verification: ', num2str(verificationT)]);
    
    time = reachabilityT; % + verificationT;
    disp(['Time needed for reachability analysis + verification: ', num2str(time)]);

%     % Verification --------------------------------------------------------
%     
%     Rset = get(HA, 'reachableSet');
%     Rset = Rset.OT;
%     
%     spec = findSafetyProperties([-0.0061322  0.064531 -0.00069603 -0.062226 -0.0035048 0.047228],Rset);
%     
%     tic;
%     safe = verifySafetyProperties(spec, Rset);
%     verificationT = toc;
%     
%     if safe
%         disp('Verification result: SAFE');
%     end
%     
%     disp(['Time needed for verification: ', num2str(verificationT)]);
%     
%     time = reachabilityT + verificationT;
%     disp(['Time needed for reachability analysis + verification: ', num2str(time)]);


% Visualization -------------------------------------------------------
if vis    
    hold on
    options.projectedDimensions = [1 2];

    options.plotType = 'b';
    plot(HA,'reachableSet',options); %plot reachable set
    plotFilled(options.R0,options.projectedDimensions,'w','EdgeColor','k'); %plot initial set
    xlabel('t');
    ylabel('x1');
    xlim([0 options.tFinal]);
    %ylim([0 0.025]);
end
end


end
